from dataclasses import dataclass
from enum import Enum

from dataclasses_json import dataclass_json


class JwtKeyType(Enum):
    PUBLIC_KEY = "public"
    PRIVATE_KEY = "private"


@dataclass_json
@dataclass
class JwtKey:
    team: str
    key: str
    payload: dict
    algorithm: str
    key_type: JwtKeyType



import os
import unittest

from yajwt.jwt_exceptions import JwtKeyNotFound
from yajwt.entities.jwt_key import JwtKeyType
from yajwt.jwt_keys_manager import JwtKeysManager


class TestJwtKeysManager(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.jwt_keys_manager: JwtKeysManager = None
        self.team_example = "testing-user"

    def setUp(self) -> None:
        keys_path = os.path.join(os.getcwd(), "examples", "keys")
        self.jwt_keys_manager = JwtKeysManager(keys_path)

    def test_get_available_private_key(self):
        jwt_key = self.jwt_keys_manager.get_private_key(self.team_example)
        self.assertEqual(self.team_example, jwt_key.team)
        self.assertEqual(JwtKeyType.PRIVATE_KEY, jwt_key.key_type)

    def test_get_not_found_private_key(self):
        not_found_team = "not-found-team"
        with self.assertRaises(JwtKeyNotFound) as e:
            _ = self.jwt_keys_manager.get_private_key(not_found_team)
        self.assertEqual(
            str(e.exception), "JwtKey not found for team '{}'".format(not_found_team)
        )

    def test_get_available_public_key(self):
        jwt_key = self.jwt_keys_manager.get_public_key(self.team_example)
        self.assertEqual(self.team_example, jwt_key.team)
        self.assertEqual(JwtKeyType.PUBLIC_KEY, jwt_key.key_type)

    def test_get_not_found_public_key(self):
        not_found_team = "not-found-team"
        with self.assertRaises(JwtKeyNotFound) as e:
            _ = self.jwt_keys_manager.get_public_key(not_found_team)
        self.assertEqual(
            str(e.exception), "JwtKey not found for team '{}'".format(not_found_team)
        )



import logging
import os
from http import HTTPStatus

from yajwt.jwt_keys_manager import JwtKeysManager
from yajwt.jwt_requests_wrapper import JwtRequestsWrapper
from yajwt.jwt_response_mapper import JwtResponseMapper

TOKEN_TTL = 60 * 10  # we recommend 10 min

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)

    # set up your public/private keys directory
    keys_path = os.path.join(os.getcwd(), "examples", "keys")
    jwt_keys_manager = JwtKeysManager(keys_path)
    jwt_requests = JwtRequestsWrapper(jwt_keys_manager, JwtResponseMapper(), TOKEN_TTL)

    # make the request
    response = jwt_requests.get("https://example.com", "testing-user")
    if response.status == HTTPStatus.OK:
        print(response)



import logging

import jwt
from jwt import ExpiredSignatureError, InvalidSignatureError

from yajwt.jwt_exceptions import JwtKeyNotFound
from yajwt.jwt_keys_manager import JwtKeysManager
from yajwt.entities.jwt_key import JwtKey
from yajwt.entities.jwt_token import JwtToken


class JwtRequestsValidator:
    def __init__(self, jwt_keys_manager: JwtKeysManager):
        self.__jwt_keys_manager = jwt_keys_manager
        self.__logger = logging.getLogger("JwtRequestsValidator")

    def validate(self, jwt_token: str) -> JwtToken:
        payload = self.__get_payload(jwt_token)
        team_name = payload.get("iss")
        if team_name is None:
            self.__logger.error("Unable to get team_name from JWT token.")
            return JwtToken(False)

        try:
            jwt_key = self.__jwt_keys_manager.get_public_key(team_name)
            return self.__validate(jwt_token, jwt_key)
        except JwtKeyNotFound as e:
            self.__logger.error(str(e))
            return JwtToken(False)

    def __get_payload(self, jwt_token: str) -> dict:
        return jwt.decode(jwt_token, verify=False)

    def __validate(self, jwt_token: str, jwt_key: JwtKey) -> JwtToken:
        try:
            jwt.decode(jwt_token, jwt_key.key, algorithms=jwt_key.algorithm)
            return JwtToken(True, self.__get_payload(jwt_token))
        except (ExpiredSignatureError, InvalidSignatureError) as e:
            self.__logger.error(f"Unable to verify {jwt_key.team} token: {str(e)}")
            return JwtToken(False)



import logging
import re
from http import HTTPStatus

import requests

from yajwt.entities.jwt_response import JwtResponse


class JwtResponseMapper:
    def __init__(self):
        self.__logger = logging.getLogger("JwtResponseMapper")

    def map(self, response: requests.Response) -> JwtResponse:
        if re.search(r"20\d", str(response.status_code)):
            return self.__get_successful_response(response)

        self.__logger.error("Request failed with '%s'", response.status_code)
        return JwtResponse(self.__get_status(response), content=response.content)

    def __get_successful_response(self, response: requests.Response) -> JwtResponse:
        status = self.__get_status(response)
        if response.content is None:
            return JwtResponse(status)
        content_type = response.headers.get("Content-Type")
        if content_type is not None and "application/json" in content_type:
            return JwtResponse(status, json=response.json())
        return JwtResponse(status, content=response.content)

    def __get_status(self, response: requests.Response) -> HTTPStatus:
        return HTTPStatus(response.status_code)



import os
import unittest

from yajwt.jwt_keys_manager import JwtKeysManager
from yajwt.jwt_requests_validator import JwtRequestsValidator


class TestJwtRequestsValidator(unittest.TestCase):
    EXPIRED_TOKEN = (
        "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJpc3MiOiJ0ZXN0aW5nLXVzZXIiLCJ2"
        "ZXJzaW9uIjoiMSIsImV4cCI6MTYwNzU5NDU5OX0.s_fuw0ut1bOOLwKgbPWDtubqO7X6c"
        "te52jtSalHrzoiMYWeWflHXbCFel9VVeLFt6oDN_Yf2xgMx3bx71W3yUQ21jjqwSCYVR5"
        "B6dw-mM15U7v-KJKbjVpBU_KOkkNyqINJAJaB6imB6zz2UG4Du68NKzlPHbCHt4VGVNIQ"
        "-4cz5kbLMkXoZmX0sO3kTqSXpW4KkB9_8IxPNGYwdaqgsVn22Hlkf9-ER8QDsn-e69Bwx"
        "fGnqf-i5J0s3uWmSvboCciE6TIYkiutH8S93rooHLJb96mglmqLu2rcH3fqr9u1hg28jG"
        "er5LRZCK1N2HsnqSGnjc1MOhnKgX5OlrHIbAg"
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.jwt_validator: JwtRequestsValidator = None

    def setUp(self) -> None:
        keys_path = os.path.join(os.getcwd(), "examples", "keys")
        jwt_keys_manager = JwtKeysManager(keys_path)
        self.jwt_validator = JwtRequestsValidator(jwt_keys_manager)

    def test_validate_expired_token(self):
        jwt_token = self.jwt_validator.validate(self.EXPIRED_TOKEN)
        self.assertEqual(jwt_token.valid, False)



from http import HTTPStatus
from typing import Optional

from dataclasses import dataclass


@dataclass(frozen=True)
class JwtResponse:
    status: HTTPStatus
    json: Optional[dict] = None
    content: Optional[bytes] = None
    exception: Optional[Exception] = None



import logging
from http import HTTPStatus
from time import time

import requests
from jwt import encode

from yajwt.jwt_exceptions import JwtKeyNotFound
from yajwt.entities.jwt_key import JwtKey
from yajwt.jwt_keys_manager import JwtKeysManager
from yajwt.jwt_response_mapper import (
    JwtResponseMapper,
    JwtResponse,
)


class JwtRequestsWrapper:
    def __init__(
        self,
        jwt_keys_manager: JwtKeysManager,
        jwt_response_mapper: JwtResponseMapper,
        expiration_time: int,
    ):
        self.__jwt_keys_manager = jwt_keys_manager
        self.__expiration_time = expiration_time
        self.__jwt_response_mapper = jwt_response_mapper
        self.__logger = logging.getLogger("JwtRequestsWrapper")

    # pylint: disable=too-many-arguments
    def get(
        self,
        url: str,
        user: str,
        params: dict = None,
        data: dict = None,
        headers: dict = None,
        timeout: int = None,
    ) -> JwtResponse:
        try:
            jwt_header = self.__build_jwt_header(user)
            headers = {**headers, **jwt_header} if headers else jwt_header

            response = requests.get(
                url, headers=headers, params=params, data=data, timeout=timeout
            )

            return self.__jwt_response_mapper.map(response)
        except (requests.exceptions.RequestException, JwtKeyNotFound) as e:
            return JwtResponse(HTTPStatus.INTERNAL_SERVER_ERROR, exception=e)

    # pylint: disable=too-many-arguments
    def post(
        self,
        url: str,
        user: str,
        params: dict = None,
        data: dict = None,
        headers: dict = None,
        files: dict = None,
        timeout: int = None,
    ) -> JwtResponse:
        try:
            jwt_header = self.__build_jwt_header(user)
            headers = {**headers, **jwt_header} if headers else jwt_header

            response = requests.post(
                url,
                headers=headers,
                params=params,
                data=data,
                files=files,
                timeout=timeout,
            )

            return self.__jwt_response_mapper.map(response)
        except (requests.exceptions.RequestException, JwtKeyNotFound) as e:
            return JwtResponse(HTTPStatus.INTERNAL_SERVER_ERROR, exception=e)

    def __build_jwt_header(self, team: str) -> dict:
        jwt_key = self.__jwt_keys_manager.get_private_key(team)
        token = self.__encode(jwt_key)
        self.__logger.info("Using JWT token: '%s'", token)
        return {"Authorization": "Bearer " + token}

    def __encode(self, jwt_key: JwtKey) -> str:
        payload = jwt_key.payload
        payload["exp"] = int(time() + self.__expiration_time)
        return encode(payload, jwt_key.key, algorithm=jwt_key.algorithm)



import os

from yajwt.jwt_keys_manager import JwtKeysManager
from yajwt.jwt_requests_validator import JwtRequestsValidator

if __name__ == "__main__":
    # set up your public/private keys directory
    keys_path = os.path.join(os.getcwd(), "examples", "keys")
    jwt_keys_manager = JwtKeysManager(keys_path)
    jwt_validator = JwtRequestsValidator(jwt_keys_manager)

    # validate needs "iss" on payload
    token = (
        "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJpc3MiOiJ0ZXN0aW5nLXVzZXIiLCJ2"
        "ZXJzaW9uIjoiMSIsImV4cCI6MTYwNzU5NDU5OX0.s_fuw0ut1bOOLwKgbPWDtubqO7X6c"
        "te52jtSalHrzoiMYWeWflHXbCFel9VVeLFt6oDN_Yf2xgMx3bx71W3yUQ21jjqwSCYVR5"
        "B6dw-mM15U7v-KJKbjVpBU_KOkkNyqINJAJaB6imB6zz2UG4Du68NKzlPHbCHt4VGVNIQ"
        "-4cz5kbLMkXoZmX0sO3kTqSXpW4KkB9_8IxPNGYwdaqgsVn22Hlkf9-ER8QDsn-e69Bwx"
        "fGnqf-i5J0s3uWmSvboCciE6TIYkiutH8S93rooHLJb96mglmqLu2rcH3fqr9u1hg28jG"
        "er5LRZCK1N2HsnqSGnjc1MOhnKgX5OlrHIbAg"
    )
    jwt_token = jwt_validator.validate(token)
    if jwt_token.valid:
        print(jwt_token.payload)
    print(jwt_token)



import json
import os
from typing import List

from yajwt.jwt_exceptions import JwtKeyNotFound
from yajwt.entities.jwt_key import JwtKey, JwtKeyType


class JwtKeysManager:
    def __init__(self, keys_path: os.path):
        self.__keys_path = keys_path
        self.__jwt_keys: List[JwtKey] = []

    def get_private_key(self, team: str) -> JwtKey:
        if not self.__jwt_keys:
            self.__jwt_keys = self.__load_keys()
        return self.__get_key(team, JwtKeyType.PRIVATE_KEY)

    def get_public_key(self, team: str) -> JwtKey:
        if not self.__jwt_keys:
            self.__jwt_keys = self.__load_keys()
        return self.__get_key(team, JwtKeyType.PUBLIC_KEY)

    def __load_keys(self) -> List[JwtKey]:
        jwt_keys = []
        if not os.path.isdir(self.__keys_path):
            raise JwtKeyNotFound("No keys found to be loaded.")

        for filename in os.listdir(self.__keys_path):
            if not filename.endswith(".json"):
                continue

            jwt_json_path = os.path.join(self.__keys_path, filename)
            jwt_json_content = self.__get_key_json_content(jwt_json_path)
            jwt_keys.append(JwtKey.from_dict(jwt_json_content))
        return jwt_keys

    def __get_key_json_content(self, key_path: os.path) -> json:
        with open(key_path, "r", encoding="UTF-8") as file:
            return json.load(file)

    def __get_key(self, team: str, key_type: JwtKeyType) -> JwtKey:
        found_jwt_key = None
        for jwt_key in self.__jwt_keys:
            if jwt_key.team == team and jwt_key.key_type == key_type:
                found_jwt_key = jwt_key
        if not found_jwt_key:
            raise JwtKeyNotFound("JwtKey not found for team '{}'".format(team))
        return found_jwt_key



"""
# Upload Package: https://pypi.org/project/yajwt/
python3 setup.py sdist bdist_wheel
twine upload dist/yajwt-0.0.4*
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="yajwt",
    version="0.0.4",
    author="Nuno Nelas",
    author_email="nuno.nelas@icloud.com",
    description="Yet Another JWT wrapper for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nnelas/yajwt",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import logging
import sys
import unittest

if __name__ == "__main__":
    logging.disable(logging.ERROR)
    loader = unittest.TestLoader()
    tests = loader.discover(".")
    testRunner = unittest.runner.TextTestRunner(verbosity=2)
    result = testRunner.run(tests)
    sys.exit(not result.wasSuccessful())
